unit CaReg;

{ by Paul Toth (TothPaul@mygale.org) }
{ http://www.mygale.org/~tothpaul    }

{

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

}

interface

Uses
 Classes,DsgnIntf,Dialogs,
 CaReg1, CaReg2, CaReg3, CaReg4,
 AS400, Cmd400, Tfr400, Dlg400, Icf400, Nof400, Call400;

Procedure Register;

implementation

Type
 THostListProperty=class(TStringProperty)
 public
  Function GetAttributes:TPropertyAttributes; override;
  Procedure GetValues(Proc:TGetStrProc); override;
  Procedure GetList(List:TStrings); virtual; abstract;
 end;

 TSysNameProperty=class(THostListProperty)
 Public
  Procedure GetList(List:TStrings); override;
 end;

 TLibNameProperty=class(THostListProperty)
 Public
  Procedure GetList(List:TStrings); override;
 end;

 TFileNameProperty=class(THostListProperty)
 Public
  Procedure GetList(List:TStrings); override;
 end;

 TMbrNameProperty=class(THostListProperty)
 Public
  Procedure GetList(List:TStrings); override;
 end;

 TAs400Editor=class(TComponentEditor)
  Function GetVerbCount:Integer; override;
  Function GetVerb(Index:Integer):String; override;
  Procedure ExecuteVerb(Index:Integer); override;
 end;

 TCmd400Editor=class(TComponentEditor)
  Function GetVerbCount:Integer; override;
  Function GetVerb(Index:Integer):String; override;
  Procedure ExecuteVerb(Index:Integer); override;
 end;

 TDlg400Editor=class(TComponentEditor)
  Function GetVerbCount:Integer; override;
  Function GetVerb(Index:Integer):String; override;
  Procedure ExecuteVerb(Index:Integer); override;
 end;

 TTfr400Editor=class(TComponentEditor)
  Function GetVerbCount:Integer; override;
  Function GetVerb(Index:Integer):String; override;
  Procedure ExecuteVerb(Index:Integer); override;
  Procedure SelectFile;
  Procedure RetreiveDescription;
 end;

 TCall400Editor=class(TComponentEditor)
  Function GetVerbCount:Integer; override;
  Function GetVerb(Index:Integer):String; override;
  Procedure ExecuteVerb(Index:Integer); override;
 end;

{ THostListProperty }

 Function THostListProperty.GetAttributes;
  begin
   Result:=[paValueList, paSortList];
  end;

 Procedure THostListProperty.GetValues(Proc:TGetStrProc);
  Var
   List:TStringList;
   i:integer;
  begin
   List:=TStringList.Create;
   GetList(List);
   For i:=0 to List.count-1 do begin
    Proc(List[i]);
   end;
   List.Free;
  end;

{ TSysNameProperty }
Procedure TSysNameProperty.GetList(List:TStrings);
 begin
  SysList(List);
  List.Insert(0,'*DEFAULT');
 end;

{ TLibNameProperty }
Procedure TLibNameProperty.GetList(List:TStrings);
 Var
  OldDoEvents:boolean;
 begin
  With TTfr400(GetComponent(0)) do begin
   OldDoEvents:=DoEvents;
   DoEvents:=False;
   LibList(List);
   DoEvents:=OldDoEvents;
  end;
  List.Insert(0,'*LIBL');
 end;

{ TFileNameProperty }
Procedure TFileNameProperty.GetList(List:TStrings);
 Var
  OldDoEvents:boolean;
 begin
  With TTfr400(GetComponent(0)) do begin
   if LibName='' then raise ETfr400.Create('LibName not set');
   OldDoEvents:=DoEvents;
   DoEvents:=False;
   FileList(LibName,List,True);
   DoEvents:=OldDoEvents;
  end;
 end;

{ TMbrNameProperty }
Procedure TMbrNameProperty.GetList(List:TStrings);
 Var
  OldDoEvents:boolean;
 begin
  With TTfr400(GetComponent(0)) do begin
   if LibName='' then raise ETfr400.Create('LibName not set');
   if FileName='' then raise ETfr400.Create('FileName not set');
   OldDoEvents:=DoEvents;
   DoEvents:=False;
   MemberList(LibName+'/'+FileName,List);
   DoEvents:=OldDoEvents;
   List.Insert(0,'*FIRST');
  end;
 end;

{ TAs400Editor }
 Function TAs400Editor.GetVerbCount;
  begin
   Result:=1;
  end;

 Function TAs400Editor.GetVerb(Index:Integer):string;
  begin
   Result:='Who am I ?';
  end;

 Procedure TAs400Editor.ExecuteVerb(Index:Integer);
  begin
   CaReg1.ShowAs400(TAs400(Component));
  end;

{ TCmd400Editor }
 Function TCmd400Editor.GetVerbCount;
  begin
   Result:=1;
  end;

 Function TCmd400Editor.GetVerb(Index:Integer):string;
  begin
   Result:='Use';
  end;

 Procedure TCmd400Editor.ExecuteVerb(Index:Integer);
  begin
   CaReg2.ShowRmtCmd(TCmd400(Component));
  end;

{ TDlg400Editor }
 Function TDlg400Editor.GetVerbCount;
  begin
   Result:=1;
  end;

 Function TDlg400Editor.GetVerb(Index:Integer):string;
  begin
   Result:='Show';
  end;

 Procedure TDlg400Editor.ExecuteVerb(Index:Integer);
  begin
   TDlg400(Component).Execute;
  end;

{ TTfr400Editor }
 Function TTfr400Editor.GetVerbCount;
  begin
   Result:=2;
  end;

 Function TTfr400Editor.GetVerb(Index:Integer):string;
  begin
   Case index of
    0: Result:='Select File';
    1: Result:='Retreive Description';
   end;
  end;

 Procedure TTfr400Editor.ExecuteVerb(Index:Integer);
  begin
   Case Index of
    0 : SelectFile;
    1 : RetreiveDescription;
   end;
  end;

 Procedure TTfr400Editor.SelectFile;
  Var
   Dlg400:TDlg400;
  begin
   Dlg400:=TDlg400.Create(nil);
   Dlg400.Host:=TTfr400(Component).Host;
   With TTfr400(Component) do begin
    if LibName<>'' then Dlg400.TableName:=LibName+'/'+FileName;
    if Dlg400.Execute then begin
     FileName:=Dlg400.FileName;
     LibName:=Dlg400.LibName;
     MbrName:='';
    end;
   end;
   Dlg400.Free;
  end;

 Procedure TTfr400Editor.RetreiveDescription;
  Var
   DDS:TDdsForm;
  begin
   DDS:=TDdsForm.Create(nil);
   DDS.SetTfr(TTfr400(Component));
   DDS.ShowModal;
   DDS.Free;
  end;

{ TCall400Editor }
 Function TCall400Editor.GetVerbCount;
  begin
   Result:=1;
  end;

 Function TCall400Editor.GetVerb(Index:Integer):string;
  begin
   Result:='Edit parms';
  end;

 Procedure TCall400Editor.ExecuteVerb(Index:Integer);
  begin
   CaReg4.EditParms(TCall400(Component));
  end;

{ Registration }

Procedure Register;
 begin
 { TAS400 }
  RegisterComponents('CAWIN',[TAS400]);
  RegisterPropertyEditor(TypeInfo(TSysName),TAs400,'SysName',TSysNameProperty);
  RegisterComponentEditor(TAs400,TAs400Editor);
 { TCmd400 }
  RegisterComponents('CAWIN',[TCmd400]);
  RegisterComponentEditor(TCmd400,TCmd400Editor);
 { TTfr400 }
  RegisterComponents('CAWIN',[TTfr400]);
  RegisterPropertyEditor(TypeInfo(TLibName),TTfr400,'LibName',TLibNameProperty);
  RegisterPropertyEditor(TypeInfo(TFileName),TTfr400,'FileName',TFileNameProperty);
  RegisterPropertyEditor(TypeInfo(TMbrName),TTfr400,'MbrName',TMbrNameProperty);
  RegisterComponentEditor(TTfr400,TTfr400Editor);
 { TDlg400 }
  RegisterComponents('CAWIN',[TDlg400]);
  RegisterComponentEditor(TDlg400,TDlg400Editor);
 { TIcf400 }
  RegisterComponents('CAWIN',[TIcf400]);
 { TCall400 }
  RegisterComponents('CAWIN',[TCall400]);
  RegisterComponentEditor(TCall400,TCall400Editor);
 { TNof400 }
  RegisterComponents('CAWIN',[TNof400]);
 end;

end.
