unit Dlg400;

{ by Paul Toth (TothPaul@mygale.org) }
{ http://www.mygale.org/~tothpaul    }

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Tfr400, As400,CaError;

type
  TDlgForm = class(TForm)
    BtnOK: TButton;
    BtnCancel: TButton;
    Label1: TLabel;
    LibList: TListBox;
    Label2: TLabel;
    FileList: TListBox;
    LibName: TLabel;
    TableName: TEdit;
    cLogical: TCheckBox;
    procedure LibListClick(Sender: TObject);
    procedure BtnCancelClick(Sender: TObject);
    procedure BtnOKClick(Sender: TObject);
    procedure FileListClick(Sender: TObject);
    procedure TableNameKeyPress(Sender: TObject; var Key: Char);
    procedure Transfer1BeforeRequest(Sender: TObject);
    procedure Transfer1AfterRequest(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private-dclarations }
    Transfer1:TTfr400;
    fLibName:string;
    fFileName:string;
    Procedure FindTable;
  public
    { Public-dclarations }
  end;

  EDlg400 = class(Exception);

  TDlg400 = class(TChild400)
  Private
   fLibName:TLibName;
   fFileName:TFileName;
   fTableName:TTableName;
   fLogicalFile:boolean;
   fDlgForm:TDlgForm;
  Public
   Constructor Create(AOwner:TComponent); override;
   Function Execute:boolean;
   Property FileName:TFileName read fFileName write fFileName;
   Property LibName:TLibName read fLibName write fLibName;
   Property TableName:TTableName read fTableName write fTableName;
  Published
   Property Logical:boolean read fLogicalFile write fLogicalFile;
  end;

var
  DlgForm: TDlgForm;

implementation

{$R *.DFM}

{ TDlg400 }
Constructor TDlg400.Create(AOwner:TComponent);
 begin
  inherited Create(AOwner);
  fDlgForm:=nil;
 end;

Function TDlg400.Execute:boolean;
 begin
  Result:=False;
  (* Note : Dlgform own Application so we don't need to Free it *)
  if fDlgForm=nil then fDlgForm:=TDlgForm.Create(Application);
  With fDlgForm do begin
   cLogical.Checked:=Logical;
   Transfer1.Host:=Self.ValidHost;
   Transfer1.DoEvents:=False;
   Transfer1.LibList(LibList.Items);
   TableName.Text:=Self.fTableName;
   fLibName:='';
   FindTable;
   Transfer1.DoEvents:=True;
   ShowModal;
   Transfer1.EndRequest;
  end;
  if fDlgForm.ModalResult=mrOk then begin
   fFileName:=fDlgForm.fFileName;
   fLibName:=fDlgForm.fLibName;
   fTableName:=fLibName+'/'+fFileName;
   Result:=True;
  end;
 end;

{ TDlgForm }

procedure TDlgForm.LibListClick(Sender: TObject);
begin
 fLibName:=LibList.Items[LibList.ItemIndex];
 fFileName:='';
 TableName.Text:=fLibName+'\';
 LibName.Caption:='\\'+Transfer1.Host.SysName+'\'+fLibName;
 FileList.Clear;
 Transfer1.FileList(fLibName,FileList.Items,cLogical.Checked);
end;

procedure TDlgForm.BtnCancelClick(Sender: TObject);
begin
 ModalResult:=mrCancel;
end;

procedure TDlgForm.BtnOKClick(Sender: TObject);
begin
 if fFileName='' then exit;
 ModalResult:=mrOk;
end;

procedure TDlgForm.FileListClick(Sender: TObject);
begin
 fFileName:=FileList.Items[FileList.ItemIndex];
 TableName.Text:=fLibName+'\'+fFileName;
end;

Procedure TDlgForm.FindTable;
 Var
  s,ss:string;
  p:byte;
  i:integer;
  Procedure GoLib(NewLib:string);
   begin
    i:=LibList.Items.IndexOf(NewLib);
    if i<0 then begin { not in the LibList }
     Try
      Transfer1.FileList(NewLib,FileList.Items,cLogical.Checked); { try to get FileList }
      fLibName:=NewLib; LibList.ItemIndex:=LibList.Items.Add(fLibName); { add Lib }
     Except
      On E:ECawin do begin ShowMessage(E.Message); s:=''; end;{ not a valid libname }
     end;
    end else begin   { existing lib }
     fLibName:=NewLib; LibList.ItemIndex:=i; { set it active }
     Transfer1.FileList(fLibName,FileList.Items,cLogical.Checked); { build file list }
    end;
    LibName.Caption:='\\'+Transfer1.Host.SysName+'\'+fLibName;
   end;
 begin
  s:=TableName.Text;
  p:=Pos('/',s);
  if p=0 then p:=Pos('\',s);
  if p>0 then begin { '/'  }
   ss:=Copy(s,1,p-1);  { extract lib name }
   if fLibName<>ss then GoLib(ss); { not the current one }
   if s<>'' then begin; {  a valid lib }
    ss:=Copy(s,p+1,Length(s)); { extract file name }
    i:=FileList.Items.IndexOf(ss);
    if i<0 then fFileName:='' else begin fFileName:=ss; FileList.ItemIndex:=i; end;
   end;
  end else
  { no '/' } if s<>'' then GoLib(s);
 end;

procedure TDlgForm.TableNameKeyPress(Sender: TObject; var Key: Char);
begin
 if Key=#13 then FindTable;
end;

procedure TDlgForm.Transfer1BeforeRequest(Sender: TObject);
begin
 Screen.Cursor:=crSQLWait;
 Enabled:=False;
end;

procedure TDlgForm.Transfer1AfterRequest(Sender: TObject);
begin
 Enabled:=True;
 Screen.Cursor:=crDefault;
end;

procedure TDlgForm.FormCreate(Sender: TObject);
begin
 Transfer1:=TTfr400.Create(Self);
 With Transfer1 do begin
  BeforeRequest := Transfer1BeforeRequest;
  AfterRequest := Transfer1AfterRequest;
  DoEvents := False;
  Conversion := cvNone;
  Translate := False;
  SupportNullVar := False;
 end;
end;

end.
