unit Icf400;

{ by Paul Toth (TothPaul@mygale.org) }
{ http://www.mygale.org/~tothpaul    }

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, AS400, CaError, Cawin;

type
  EIcf400 = class(Exception);

  TICFStatus=set of (hsReceiving);

  TIcf400 = class(TChild400)
  private
    { Private-dclarations }
    fActive:boolean;
    fPartner:string;
    ICFId:LongInt;
    Status:TICFStatus;
    Receive:Integer;
    Procedure SetActive(Value:boolean);
    Procedure SetPartner(Value:string);
  protected
    { Protected-dclarations }
    Destructor Destroy; override;
    Procedure Loaded; override;
    procedure InternalError; override;
    Procedure StartICF;
    Procedure StopICF;
  public
    { Public-dclarations }
    Function Request(Var Rqst; RqstSize:Word; Var Reply; ReplySize:Word):word;
    Procedure SendData(Var Data; DataSize:Word);
    Function ReceiveAndWait(Var Data; DataSize:Word):word;
  published
    { Published-dclarations }
    Property Active:boolean read fActive write SetActive;
    Property Partner:string read fPartner write SetPartner;
  end;


procedure Register;

implementation

Const
{$DEFINE FR}
{$IFDEF FR}
 ICF000:string='Erreur ICF $';
 ICF001:string='Impossible car actif';
 ICF003:string='Erreur de lecture';
 ICF004:string='Lecture en cours';

 ICF015:string='Communication invalide';
 ICF033:string='Communication impossible';
 ICF035:string='Programme cible non trouv';
 ICF046:string='Programme cible interrompu';
 ICF051:string='Communication interrompue';
{$ELSE}
 ICF000:string='ICF error $';
 ICF001:string='Impossible will actif';
 ICF003:string='Read error';
 ICF004:string='Reading';
 ICF015:string='Invalide communication';
 ICF033:string='Allocation error';
 ICF035:string='Target program not found';
 ICF046:string='Target program aborted';
 ICF051:string='Communication aborted';
{$ENDIF}

{ ehnappc }
 Mapped = 1;
 NoSync = 0;
 Fill   = 11;
 ReceiveComplet = 1;
 Received       = 6;

{ TICF400 }

Type
 THeader=record
  Size,GDSID:word;
 end;

Destructor TIcf400.Destroy;
 begin
  StopICF;
  Inherited Destroy;
 end;

Procedure TIcf400.InternalError;
 begin
  raise ECawin.CreateCAGet('RTR',Error);
 end;

Procedure TIcf400.SetActive(Value:boolean);
 begin
  if fActive<>Value then
   if (csLoading in ComponentState) then
    fActive:=Value
   else
    if Value then StartICF else StopICF;
 end;

Procedure TIcf400.SetPartner(Value:string);
 begin
  if fPartner=Value then exit;
  if Active and not (csLoading in ComponentState) then raise EIcf400.Create('Invalid while Active');
  fPartner:=Value;
 end;

Procedure TIcf400.Loaded;
 begin
  Inherited Loaded;
  If Active then StartICF;
 end;

Procedure TIcf400.StartICF;
 begin
  fActive:=False;
  fPartner[Length(fPartner)+1]:=#0; { PChar }
  Error:=EHNAPPC_Allocate(hInstance,1024,Mapped,NoSync,ValidHost.SysNameZ,@fPartner[1],0,nil,IcfID);
  fActive:=True;
 end;

Procedure TIcf400.StopICF;
 begin
  fActive:=False;
  Error:=EHNAPPC_DeAllocate(hInstance,IcfId,Mapped);
 end;

Function TIcf400.Request(Var Rqst; RqstSize:Word; Var Reply; ReplySize:Word):word;
 begin
  SendData(Rqst,RqstSize);
  Result:=ReceiveAndWait(Reply,ReplySize);
 end;

Procedure TICF400.SendData(Var Data; DataSize:Word);
 Var
  Header:THeader;
  Rqst:byte;
 begin
  If hsReceiving in Status then raise EICF400.Create('Send invalid while Reading');
  With Header do begin
   size:=swap(DataSize+SizeOf(THeader));
   GDSID:=$FF12;
  end;
  Error:=EHNAPPC_SendData(hInstance,ICFId,SizeOf(THeader),Header,Rqst);
  Error:=EHNAPPC_SendData(hInstance,ICFId,DataSize,Data,Rqst);
 end;

Function TICF400.ReceiveAndWait(Var Data; DataSize:Word):Word;
 Var
  Header:THeader;
  What,Rqst:byte;
  Dummy:byte;
  ReceiveDummy:word;
 begin
  Error:=EHNAPPC_ReceiveAndWait(hInstance,ICFId,Fill,SizeOf(THeader),Header,What,Rqst,ReceiveDummy);
  Receive:=Swap(Header.Size)-SizeOf(Header);
  Error:=EHNAPPC_ReceiveAndWait(hInstance,ICFId,Fill,DataSize,Data,What,Rqst,Result);
  Dec(Receive,Result);
  If what=ReceiveComplet then begin
   Error:=EHNAPPC_ReceiveAndWait(hInstance,ICFId,Fill,0,Dummy,What,Rqst,ReceiveDummy);
   if What<>Received then raise EIcf400.Create(ICF003);
   Exclude(Status,hsReceiving);
  end else
   Include(Status,hsReceiving);
 end;

{ Registration }

procedure Register;
begin
  RegisterComponents('AS/400', [TIcf400]);
end;

end.
