unit Resource;

{ by Paul Toth (TothPaul@mygale.org) }
{ http://www.mygale.org/~tothpaul    }

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls;

type
 { This component store StringTable resources (not as STRINGTABLE but like
   delphi, in a RCDDATA) }
  TResource=class(TComponent)
  private
   fStringTable:TStringList;
   Procedure SetStringTable(Value:TStringList);
  Public
   Constructor Create(AOwner:TComponent); override;
   Destructor Destroy; override;
  Published
   Property StringTable:TStringList read FStringTable write SetStringTable;
  end;

{ This component is a TLabel that take it's caption from a TResource stringtable }
  TResCaption = class(TLabel)
  private
    { Dclarations private }
    fResource:TResource;
    fStringName:string;
    procedure SetResource(Value:TResource);
    procedure SetStringName(Value:string);
    Procedure GetValue;
    procedure WMUser(Var Msg:TMessage); message WM_USER;
  protected
    { Dclarations protected }
  public
    { Dclarations public }
    Procedure Loaded; override;
  published
    { Dclarations published }
    Property Resource:TResource read fResource write SetResource;
    Property StringName:string read fStringName write SetStringName;
    Property Caption stored false;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Additional', [TResource, TResCaption]);
end;

{ TResource }

Constructor TResource.Create(AOwner:TComponent);
 begin
  inherited Create(AOwner);
  fStringTable:=TStringList.Create;
 end;

Destructor TResource.Destroy;
 begin
  fStringTable.Free;
  inherited Destroy;
 end;

procedure TResource.SetStringTable(Value:TStringList);
 Var
  Msg:TMessage;
 begin
  fStringTable.Assign(Value);
  Msg.Msg:=WM_USER;
  Msg.WParam:=0;
  Msg.LParam:=LongInt(Self);
  Msg.Result:=0;
  TWinControl(Owner).BroadCast(Msg);
 end;

{ TResCaption }

Procedure TResCaption.Loaded;
 begin
  inherited Loaded;
  GetValue;
 end;

Procedure TResCaption.WMUser(Var Msg:TMessage);
 begin
  if TComponent(Msg.LParam)=FResource then GetValue;
  inherited;
 end;

Procedure TResCaption.SetStringName(Value:string);
 begin
  if fStringName<>Value then begin
   fStringName:=Value;
   GetValue;
  end;
 end;

procedure TResCaption.SetResource(Value:TResource);
 begin
  if fResource<>Value then begin
   fResource:=Value;
   GetValue;
  end;
 end;

Procedure TResCaption.GetValue;
 begin
  if FResource=nil then
   Caption:=Name
  else begin
   Caption:=FResource.StringTable.Values[fStringName];
   if Caption='' then Caption:=FResource.Name;
  end;
 end;

end.
